#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

# import standard libraries

# import own libraries
import weightanalyser.dataanalyser as dataanalyser


def generate_testdataset():
    """docstring for generate_testdataset"""
    dataset = { "general" : { "start_date" : "01-09-2016" }, "datapoints" : [] }

    for i_datapoint in range(40):
        datapoint = {}
        datapoint.update( { "date" : dataanalyser.get_date_from_number(i_datapoint,
                                                                       dataset["general"]["start_date"]) })
        datapoint.update( { "weight" : 100 })
        datapoint.update( { "fat_perc" : 20 } )
        datapoint.update( { "fat_mass" : 20 } )
        datapoint.update( { "mus_perc" : 40 } )
        datapoint.update( { "mus_mass" : 40 } )

        dataset["datapoints"].append(datapoint)

    return dataset


def test_generate_testdataset():
    """docstring for test_generate_testdataset"""
    testdataset = generate_testdataset()
    assert testdataset["datapoints"][0]["date"] == "01-09-2016"
    assert testdataset["datapoints"][0]["weight"] == 100
    assert testdataset["datapoints"][0]["fat_perc"] == 20
    assert testdataset["datapoints"][0]["fat_mass"] == 20
    assert testdataset["datapoints"][0]["mus_perc"] == 40
    assert testdataset["datapoints"][0]["mus_mass"] == 40


def test_calc_parameters():
    """docstring for test_generate_testdataset"""
    testdataset = generate_testdataset()
    parameters = dataanalyser.calc_parameters(testdataset, 14)
    assert parameters["weights_parameters"][0] == 0
    assert parameters["weights_parameters"][1] == 100
    assert parameters["fat_masses_parameters"][0] == 0
    assert parameters["fat_masses_parameters"][1] == 20
    assert parameters["mus_masses_parameters"][0] == 0
    assert parameters["mus_masses_parameters"][1] == 40


def test_get_date_from_number():
    assert dataanalyser.get_date_from_number(5, "01-08-2016") == "06-08-2016"

def test_get_number_from_date():
    """docstring for test_get_number_from_date"""
    assert dataanalyser.get_number_from_date("06-08-2016", "01-08-2016") == 5


def test_get_index_of_period():
    """docstring for test_get_index_of_period"""
    dataset = generate_testdataset()
    period = 14
    assert dataanalyser.get_index_of_period(dataset, 10) == 28



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path
import re

import weightanalyser.datahandeling as datahandeling


def add_measurement(measurement):
    """docstring for add_measurement"""
    path = os.path.expanduser("~") + "/Dropbox/weightanalyser/data/dataset.js"
    if os.path.isfile(path):
        dataset = datahandeling.read_dataset(path)
        dataset["datapoints"].append(measurement)
    else:
        dataset = {
                    "general": {"start_date": measurement["date"]},
                    "datapoints": [measurement]}
    datahandeling.write_dataset(path, dataset)

    return dataset


def check_date_format(date_string):
    if re.search('[a-zA-Z]', date_string):
        return "Contains letter!"
    if not re.search("..-..-....", date_string):
        return False
    else:
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path

import matplotlib.pyplot as plt

import weightanalyser.dataanalyser as dataanalyser


def make_function_dataset(slope, intersec, start_date_number, end_date_number):
    """docstring for make_function_dataset"""
    function_dataset = [[],[]]
    
    for i in range(start_date_number, end_date_number):
        function_dataset[0].append(i)
        function_dataset[1].append(i * slope + intersec)

    return function_dataset
    
def make_plot(dataset):
    """docstring for make_plot"""
    parameters = dataanalyser.calc_parameters(dataset, 14)
    dataset_start_date = dataset["general"]["start_date"]
    start_index = parameters["start_index"]
    start_date_number = dataanalyser.get_number_from_date(dataset["datapoints"][start_index]["date"], dataset_start_date)
    last_index = parameters["last_index"]
    datenumbers = []
    weights = []
    fat_masses = []
    mus_masses = []
    for i in range(len(dataset["datapoints"])):
        datenumbers.append(
                            dataanalyser.get_number_from_date(
                                dataset["datapoints"][i]["date"],
                                dataset["general"]["start_date"]))
        weights.append(dataset["datapoints"][i]["weight"])
        fat_masses.append(dataset["datapoints"][i]["fat_mass"])
        mus_masses.append(dataset["datapoints"][i]["mus_mass"])

    f, (ax1, ax2, ax3) = plt.subplots(3, sharex=True)

    ax1.set_title("Full Mass")
    ax1.set_ylabel("full mass [kg]")
    ax1.grid()
    full_change_per_week = parameters["weights_parameters"][0] * 7
    full_change_per_day = parameters["weights_parameters"][0]
    full_intersec = parameters["weights_parameters"][1]
    textstr1 = "%.2f kg total / week" % (full_change_per_week)
    props = dict(boxstyle='round', facecolor='white')
    ax1.text(
                0.755, 0.95, textstr1, transform=ax1.transAxes,
                fontsize=10, verticalalignment='top', bbox=props)
    ax1.plot(datenumbers, weights)
    
    weight_function_dataset = make_function_dataset(parameters["weights_parameters"][0],
                                                parameters["weights_parameters"][1],
                                                start_date_number,
                                                last_index)
    ax1.plot(weight_function_dataset[0], weight_function_dataset[1])

    ax2.plot(datenumbers, fat_masses)
    ax2.set_title("Fat Mass")
    ax2.set_ylabel("fat mass [kg]")
    ax2.grid()
    fat_change_per_week = parameters["fat_masses_parameters"][0] * 7
    textstr2 = "%.2f kg fat / week" % (fat_change_per_week)
    textstr2 += "\n= %.2f kcal / day" % (fat_change_per_week * 1000)
    props = dict(boxstyle='round', facecolor='white')
    ax2.text(
                0.755, 0.95, textstr2, transform=ax2.transAxes, fontsize=10,
                verticalalignment='top', bbox=props)

    fat_function_dataset = make_function_dataset(parameters["fat_masses_parameters"][0],
                                                parameters["fat_masses_parameters"][1],
                                                start_date_number,
                                                last_index)
    ax2.plot(fat_function_dataset[0], fat_function_dataset[1])


    ax3.plot(datenumbers, mus_masses)

    ax3.set_title("Muscle Mass")
    ax3.set_ylabel("muscle mass [kg]")
    ax3.grid()
    ax3.set_xlabel(
            "Days since start date (" + dataset["general"]["start_date"] + ")")
    muscle_change_per_week = parameters["mus_masses_parameters"][0] * 7
    textstr3 = "%.2f kg muscle / week" % (muscle_change_per_week)
    props = dict(boxstyle='round', facecolor='white')
    ax3.text(
                0.01, 0.95, textstr3, transform=ax3.transAxes, fontsize=10,
                verticalalignment='top', bbox=props
            )

    mus_function_dataset = make_function_dataset(parameters["mus_masses_parameters"][0],
                                                parameters["mus_masses_parameters"][1],
                                                start_date_number,
                                                last_index)
    ax3.plot(mus_function_dataset[0], mus_function_dataset[1])

    path = os.path.expanduser("~") + "/Dropbox/weightanalyser/data/plot.png"
    plt.savefig(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random
from scipy import stats
from datetime import datetime
from datetime import timedelta


def get_date_from_number(number, start_date):
    """docstring for get_date_from_number"""
    date_format = "%d-%m-%Y"
    start_date_object = datetime.strptime(start_date, date_format)
    number_date_object = start_date_object + timedelta(days=number)

    return number_date_object.strftime(date_format)


def get_number_from_date(date, start_date):
    """docstring for get_number_from_date"""
    date_format = "%d-%m-%Y"
    time_difference = datetime.strptime(date, date_format) - \
            datetime.strptime(start_date, date_format)

    return time_difference.days


def get_index_of_period(dataset, period):
    length_dataset = len(dataset["datapoints"])
    i = length_dataset - 1
    dataset_period = 0
    while dataset_period < period and i != 0:
        dataset_period = get_number_from_date(dataset["datapoints"][length_dataset-1]["date"], dataset["datapoints"][i]["date"])
        i -= 1
    return i


def calc_parameters(dataset, period):
    """docstring for get_weightparameters"""
    datenumbers = []
    weights = []
    fat_masses = []
    mus_masses = []

# only take the last time_period days for the linear regression
    last_date = dataset["datapoints"][len(dataset["datapoints"])-1]["date"]
    last_index = get_number_from_date(last_date, dataset["general"]["start_date"])+1
    start_index = get_index_of_period(dataset, period)


    for i_datapoint in range(start_index, len(dataset["datapoints"])-1):
        datenumbers.append(get_number_from_date(dataset["datapoints"][i_datapoint]["date"], dataset["general"]["start_date"]))
        weights.append(dataset["datapoints"][i_datapoint]["weight"])
        fat_masses.append(dataset["datapoints"][i_datapoint]["fat_mass"])
        mus_masses.append(dataset["datapoints"][i_datapoint]["mus_mass"])

    weights_parameters =  stats.linregress(datenumbers, weights)
    fat_masses_parameters =  stats.linregress(datenumbers, fat_masses)
    mus_masses_parameters =  stats.linregress(datenumbers, mus_masses)

    return { "weights_parameters" : weights_parameters,
            "fat_masses_parameters" : fat_masses_parameters,
            "mus_masses_parameters" : mus_masses_parameters,
            "start_index" : start_index,
            "last_index" : last_index}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

# import standard libraries
import os

import weightanalyser.datahandeling as datahandeling

def test_writeread():
    """Test the wirte_datapoint and read_dataset functions of the datahandling module.

    It just writes a datapoint to a dataset in a file and reads the dataset afterwards.
    Because the datapoint is the only one in the dataset, the first index of the dataset
    should be equal to the datapoint.
    """
    datapoint = { "date" : "2016-09-13", "weight" : 90, "fat_perc" : 33, "mus_perc" : 33 }
    path = "test/testdata/test_writeread.dat"
    if os.path.exists(path):
            os.remove(path)
    datahandeling.write_dataset(path, datapoint)
    dataset = datahandeling.read_dataset(path)
    assert datapoint == dataset



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
import unittest

import weightanalyser.cli as cli


class Test_check_date_format(unittest.TestCase):
    def test_check_date_format_for_letters(self):
        assert cli.check_date_format("a") == "Contains letter!"

    def test_check_date_format_right(self):
        assert cli.check_date_format("01-12-2016")

    def test_check_date_format_wrong_format(self):
        self.assertFalse(cli.check_date_format("01--2016"))
        self.assertFalse(cli.check_date_format("01-2016"))
        self.assertFalse(cli.check_date_format("-01-2016"))
        self.assertFalse(cli.check_date_format("01-1-2016"))
        self.assertFalse(cli.check_date_format("1-01-2016"))
        self.assertFalse(cli.check_date_format("01-01-16"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import weightanalyser.dataanalyser as dataanalyser


def generate_testdataset():
    """docstring for generate_testdataset"""
    dataset = { "general" : { "start_date" : "01-09-2016" }, "datapoints" : [] }

    for i_datapoint in range(40):
        datapoint = {}
        datapoint.update( { "date" : dataanalyser.get_date_from_number(i_datapoint,
                                                                       dataset["general"]["start_date"]) })
        datapoint.update( { "weight" : 100 })
        datapoint.update( { "fat_perc" : 20 } )
        datapoint.update( { "fat_mass" : 20 } )
        datapoint.update( { "mus_perc" : 40 } )
        datapoint.update( { "mus_mass" : 40 } )

        dataset["datapoints"].append(datapoint)

    return dataset


#def test_make_plot():
#    """docstring for test_make_plot"""
#    dataset = generate_testdataset()
#    parameters = dataanalyser.calc_parameters(dataset, 14)
#    assert parameters["last_index"] == len(dataset["datapoints"])-1
#    visualisation.make_plot(dataset)
#    



from setuptools import setup, find_packages

setup(name='weightanalyser',
      version='0.1',
      description='Analyse your weight in depth',
      url='https://github.com/finn-christiansen/weightanalyser',
      author='Finn Christiansen',
      author_email='mail@finnchristiansen.com',
      packages=find_packages(exclude="test"),
      scripts = [
                  'scripts/weightanalyser'
                      ])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# import standard libraries
import json
import os


def read_dataset(path):
    f = open(path, "r")
    dataset = json.loads(f.read())
    f.close()
    return dataset


def write_dataset(path, dataset):
    f = open(path, "w")
    json.dump(dataset, f, indent=2)
    f.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from datetime import date

import weightanalyser.cli as cli
import weightanalyser.datahandeling as datahandeling
import weightanalyser.visualisation as visualisation

path = os.path.expanduser("~") + "/Dropbox/weightanalyser/data/dataset.js"

print "Hello, this is the WeightAnalyser!"
date_format = "%d-%m-%Y"

def run():
    do_add = "y"
    while do_add == "y":
        do_add = raw_input("Do you want to add a new measurement? [y/n] ")
        if do_add == "y":
            before_today = raw_input("Is the measurement taken before today? [y/n] ")
            if before_today == "y":
                date_str = raw_input("Please enter the date you took the measurement (dd-mm-yyyy)! ")
            else:
                date_object = date.today()
                date_str = date_object.strftime(date_format)

            print date_str
            weight = float(raw_input("Enter full weight in kg! "))
            fat_perc = float(raw_input("Enter fat perc! "))
            mus_perc = float(raw_input("Enter mus perc! "))

            measurement = { "date" : date_str,
                            "weight" : float(weight),
                            "fat_perc" : float(fat_perc),
                            "fat_mass" : float(weight * fat_perc / 100.0),
                            "mus_perc" : float(mus_perc),
                            "mus_mass" : float(weight * mus_perc / 100.0) }

            dataset = cli.add_measurement(measurement)
        else:
            dataset = datahandeling.read_dataset(path)

    visualisation.make_plot(dataset)




